package refTests;

import org.apache.log4j.xml.DOMConfigurator;

import fr.cnes.celestlab.celestlabx.area.Area;
import fr.cnes.celestlab.celestlabx.area.AreaScene;
import fr.cnes.celestlab.celestlabx.generic.Information;
import fr.cnes.celestlab.celestlabx.generic.Params;
import fr.cnes.celestlab.celestlabx.stela.Conversion;
import fr.cnes.celestlab.celestlabx.stela.Derivation;
import fr.cnes.celestlab.celestlabx.stela.Extrapolation;
import fr.cnes.celestlab.celestlabx.stela.ExtrapolationResults;
import fr.cnes.los.stela.commons.logic.log.service.LoggerAgentFactory;
import fr.cnes.los.stela.commons.logic.log.service.LoggerException;
import fr.cnes.los.stela.commons.logic.message.CommonMessage;
import fr.cnes.los.stela.commons.model.Prop;

public class RefTests {

    protected static final String SYSTEM_SEPARATOR = System.getProperty("file.separator");
    protected static final String STELA_DATA_DIR = "data" + SYSTEM_SEPARATOR + "stela" + SYSTEM_SEPARATOR;
    
	final String root = STELA_DATA_DIR;

	// Default parameters
	InitParams initParams = new InitParams();
	Params params = initParams.getParams();
	Params params_backup = initParams.getParams();

	double cjd0 = 20000;
	double[] x0 = { 24678136.0, 0.7172341, 0.0, 0.0874887, 0.0, 0.0 };

	double[] delta_t = { 0, 86400 * 2, 86400 * 4, 86400 * 6, 86400 * 8, 86400 * 10 };
		
	// Time scale offsets: TT - TREF and UT1 - TREF (sec)
	double ut1_tref = 10;
	double tt_tref = 67.184;

	int res = 0;
	
	// STELA initialization indicator 
	static boolean init_ok = false; 
	
	public static enum Typ {
		tumb, spin, fix, fixr;
	}

	public double[] CLx_stela_area(AreaScene desc, Typ typ, double[][] dirs,
			int nbp, String[] ids, double[][] axes, double[] angs,
			double[][] cens) throws Exception {

		Area area = new Area();
		area.setScene(desc);

		double[] result = null;

		switch (typ) {

		case tumb:
			Information infoTumb = area.computeTumbling(nbp);
			checkInfo(infoTumb);
			result = area.getValue();
			break;

		case spin:
			Information infoSpin = area.computeSpinning(dirs, axes, nbp);
			checkInfo(infoSpin);
			result = area.getValue();
			break;

		case fix:
			Information infoFix = area.computeFixedDir(dirs);
			checkInfo(infoFix);
			result = area.getValue();
			break;

		case fixr:
			Information infoFixRot = area.computeFixedDirRot(dirs, ids, axes, angs, cens);
			checkInfo(infoFixRot);
			result = area.getValue();
			break;
		}
		return result;
	}

	public double[][] CLx_stela_convert(String cmd, double[] cjd, double[][] x,
			Params params, double ut1_tref, double tt_tref) throws Exception {

		// Convert to osculating elements
		Conversion conv = new Conversion();
		Information infoConv = conv.run(cmd, cjd, x, params, ut1_tref, tt_tref);
		
		checkInfo(infoConv);

		double[][] convVal = conv.getResults();

		return convVal;
	}

	public double[][] CLx_stela_deriv(double[] cjd, double[][] x,
			Params params, int res, double ut1_tref, double tt_tref) throws Exception {
		Derivation derivation = new Derivation();

		Information infoDev = derivation.run(cjd, x, params, res, ut1_tref, tt_tref);
		
		checkInfo(infoDev);

		double[][] xdot = derivation.getDerivativeMatrix();

		return xdot;

	}

	public ExtrapolationResults CLx_stela_extrap(double cjd0, double[] x0,
			double[] delta_t, Params params, int res, double ut1_tref,
			double tt_tref) throws Exception {
		Extrapolation extrapolation = new Extrapolation();

		Information info = extrapolation.run(cjd0, x0, delta_t, params, res, ut1_tref, tt_tref);

		checkInfo(info);
		
		ExtrapolationResults results = extrapolation.getExtrapolationResults();

		return results;
	}

	public void initStela()  {

		if (!init_ok) {
        Prop.defineROOT(root);
        DOMConfigurator.configure(ClassLoader.getSystemResource("stela_console_log4j.xml"));

        try {
            LoggerAgentFactory.getLoggerAgent().init("stela_service_log_config.xml");
            LoggerAgentFactory.getLoggerAgent().newLogger("StelaMainLog");
        } catch (final LoggerException e) {
            System.err.println("Error during initialize logger");
            System.exit(2);
        }

        final fr.cnes.los.stela.commons.logic.log.service.Logger logger = LoggerAgentFactory.getLoggerAgent().getDefaultLogger();

        logger.log(CommonMessage.initialization());
		}
		init_ok = true; 

	}
	
	public void checkInfo(final Information info) throws Exception {
		if (info.getStatus() != 0) {
			throw new Exception(info.getMessage());
		}
	}
}
